@@ -2,7 +2,8 @@
 # and returns the customized copy of <network>
 fixGenes <- function(network,fixIndices,values)
 {
-  stopifnot(inherits(network,"BooleanNetwork") | inherits(network,"ProbabilisticBooleanNetwork") | inherits(network,"BooleanNetworkCollection"))
+  stopifnot(inherits(network,"BooleanNetwork") | inherits(network,"SymbolicBooleanNetwork") | 
+            inherits(network,"ProbabilisticBooleanNetwork"))
 
   if (length(fixIndices) != length(values) && length(values) != 1)
     stop("fixIndices and values must have the same number of elements, or values must have 1 element!")
@@ -13,6 +14,10 @@ fixGenes <- function(network,fixIndices,values)
   if (any(values != 0 & values != 1 & values != -1))
     stop("Please supply only 0, 1, or -1 in values!")
 
-  network$fixed[fixIndices] <- values
+  network$fixed[fixIndices] <- as.integer(values)
+  
+  if (inherits(network,"SymbolicBooleanNetwork"))
+    network$internalStructs = .Call("constructNetworkTrees",network);
+    
   return(network)
 }
